import java.io.*;
import java.util.*;

/** Verifier for Basic Authorization. */
class AuthorizationVerifier {
    private Map auth;

    /** Creates a new AuthorizationVerifier from the aut.cfg file passed
     * as parameter. */
    public AuthorizationVerifier(String file) {
	auth = new HashMap(); String line, token, url, passwd;
	try {
	    LineNumberReader lnr = new LineNumberReader(new FileReader(file));
	    while ( (line = lnr.readLine()) != null) {
		StringTokenizer st = new StringTokenizer(line);
		while (st.hasMoreTokens()) 
		    if ((token=st.nextToken()).equals("url:")) {
			url = st.nextToken();
			url.trim();
			HashSet sh = new HashSet();
			while (!(line = lnr.readLine()).startsWith(":lru")) {
			    line = line.trim();
			    sh.add(line);
			}
			auth.put(url, sh);
		    }
	    }
	}
	catch (IOException ioe) {}
    }
    
    /** Verifies if an URL needs to be authorized for access. */
    boolean authRequired(String url) {
	if (auth.containsKey(url)) return true;
	else return false;
    }

    /** Checks for a password. */
    boolean passwdVerify(String url, String passwd) {
	HashSet hs = (HashSet) auth.get(url);
	if (hs == null) return false;
	if (hs.contains(passwd)) return true;
	else return false;
    }

    /** Sends to the OutputStream an HTTP message which informs that the 
	authentication procedure has failed. */
    public static void warnAuthRequired(OutputStream os, String realm) {
	String warnHeader = "HTTP/1.0 401 Unauthorized -- authentication failed\n" +
	    "Server: RAMM Proxy v1.0\n" +
	    "Accept-Ranges: bytes\n" +
	    "Content-Length: 0\n" +
	    "WWW-Authentication: Basic realm=" + realm + "\n"+
	    "\n\r";
	try {
	    os.write(warnHeader.getBytes());
	} catch (IOException e) { System.out.println(e); }	
    }

    /** Sends to the OutputStream an HTML page which informs that the 
	authentication procedure has failed. */
    public static void warnForbidden(OutputStream os) {
	String warnMsg = "<HTML>\n<HEAD>\n" + 
	    "<TITLE>Access Denied</TITLE>\n" +
	    "</HEAD>\n<BODY>\n" +
	    "<H1>Access Denied</H1>\n<BR>" +
	    "error 403: FORBIDDEN\n" + 
	    "<BR><HR>\n" +
	    "<I>Generated by RAMM Proxy at " + new Date() + "</I>\n" +
	    "</BODY></HTML>\n";
	String warnHeader = "HTTP/1.0 403 Forbidden -- authentication failed\n" +
	    "Server: RAMM Proxy v1.0\n" +
	    "Accept-Ranges: bytes\n" +
	    "Content-Length: " + warnMsg.length() + "\n" +
	    "\n\r";
	try {
	    os.write((warnHeader + warnMsg).getBytes());
	} catch (IOException e) { System.out.println(e); }	
    }

}



