import java.io.*;
import java.util.*;


/**
 * Configuration of the proxy.
 */
public class Configuration {

    /**
     * Reads the configuration from file.
     * @param cfgFile  the name of the configuration file
     */
    public Configuration(String cfgFile) {
	try {
	    FileInputStream fis = new FileInputStream(new File(cfgFile));
	    Properties prop = new Properties();
	    prop.load(fis);
	    
	    ProxyMain.MIN_THREADS = 
		Integer.parseInt(prop.getProperty("min-threads"));
	    ProxyMain.MIN_SPARE_THREADS = 
		Integer.parseInt(prop.getProperty("min-spare-threads"));
	    ProxyMain.MAX_THREADS = 
		Integer.parseInt(prop.getProperty("max-threads"));
	    ProxyMain.MAX_SPARE_THREADS = 
		Integer.parseInt(prop.getProperty("max-spare-threads"));
	    ProxyMain.PROXY_PORT = 
		Integer.parseInt(prop.getProperty("port"));
	    ProxyMain.LOGFILE =
		prop.getProperty("log-filename");
	    if (prop.getProperty("enable-logging").equals("no"))
		ProxyMain.ENABLE_LOGGING = false;
	    else
		ProxyMain.ENABLE_LOGGING = true;
	    ProxyMain.CONTROLPANEL_UPDATE_DELAY =
		Integer.parseInt(prop.getProperty(
					   "update-controlpanel-every-nsecs"));
	} catch (IOException e) {
	    System.out.println("Warning: error reading configuration file " +
			       cfgFile + "\n" +
			       "Using default values instead.");
	    setDefaults();
	}
    }


    /**
     * Sets defaults for proxy values.
     */
    public static void setDefaults() {
	ProxyMain.MIN_THREADS = 7;
	ProxyMain.MIN_SPARE_THREADS = 5;
	ProxyMain.MAX_THREADS = 50;
	ProxyMain.MAX_SPARE_THREADS = 5;
	ProxyMain.PROXY_PORT = 1080;
	ProxyMain.LOGFILE = "proxy.log";
	ProxyMain.ENABLE_LOGGING = true;
	ProxyMain.CONTROLPANEL_UPDATE_DELAY = 20;
    }
	
    



   



}
