import java.net.*;
import java.io.*;
import java.util.*;


/**
 * Stream reader for HTTP requests.
 * The start line contains the whole URL (http://host/file) to retrieve. 
 */
public class HTTPRequestReader extends HTTPMessageReader {

    /** Start line: The method, eg. GET, POST, etc. */
    String command;

    /** Start line: The host concerned in the request */
    String host;

    /** Start line: The file (with full path) concerned in the request */
    String file;

    /** The whole start line */
    String startLine;

    /** The Basic authorization parameter */
    String auth;

    /**
     * Retrieves a HTTP request message from the connection stream.
     * @param istream  the stream
     */
    public HTTPRequestReader(InputStream istream) 
	                          throws IOException, NoSuchElementException {
	DataInputStream distream = new DataInputStream(istream);
	this.contentLength = 0;
	retrieveHeader(distream);

	// We've got the header, let's parse it

	// Parses the start line
	StringTokenizer st = 
	    new StringTokenizer(new String(HTTPMessageReader.toArray(header)));
	command = st.nextToken();
	URL url = new URL(st.nextToken());
	host = url.getHost();
	file = url.getFile();
	versionProtocol = st.nextToken();

	startLine = new String(command + " " + url + " " + versionProtocol);

	// Parses, within the remaining header lines, the fields we want
	//control:
	String s;
	while (st.hasMoreTokens()) {
	    s = st.nextToken();
	    if (s.equals("Content-Length:"))
		contentLength = Integer.parseInt(st.nextToken());
	    if (s.equals("Connection:")) {
		connection = new String(st.nextToken());
		if (connection.equals("keep-alive")) mustCloseConnection = false;
	    }
	    if (s.equals("Authorization:")) {
		s = st.nextToken();
		if (s.equals("Basic"))  
		    auth = new String(st.nextToken());
	    }
	}

	retrieveBody(distream);
    }
}
