import java.io.*;
import java.util.*;
import java.text.*;


/**
 * Manager for the proxy logfile.
 * Independent from the rest of program.
 */
public class LogfileManager {

    public final static int SORT_BY_CLIENT = 0;
    public final static int SORT_BY_DATE = 1;
    public final static int SORT_BY_REQUEST = 2;
    public final static int SORT_BY_RESULT = 3;


    private String filename;
    private Date filedate;

    private ArrayList client;
    private ArrayList date;
    private ArrayList request;
    private ArrayList result;

    private int nLines;


    public LogfileManager(String filename) {
	initialize();
	this.filename = filename;
    }


    /** Initializes all fields. */
    private void initialize() {
	client = new ArrayList();
	date = new ArrayList();
	request = new ArrayList();
	result = new ArrayList();
    }
    

    /**
     * Writes a line onto the logfile.
     * @param action  the line to write on the logfile
     */
    public synchronized void writeLine(String action) throws IOException {
	if (ProxyMain.TRACEON_LOG) 
	    System.out.println(action);
	FileWriter fw = new FileWriter(filename, true);  // append mode
	fw.write(action + "\n");
	fw.close();
    }


    /** 
     * Reads a logfile in memory.
     */
    public void read() throws FileNotFoundException, IOException {
	String line;
	initialize();

	// Gets datestamp from logfile
	filedate = new Date(new File(filename).lastModified());

	// Reads every line from logfile
	LineNumberReader lnr = new LineNumberReader(new FileReader(filename));
	while ((line = lnr.readLine()) != null) {

	    // puts the informations in the ArrayLists
	    StringTokenizer st = new StringTokenizer(line, "]");

	    //System.out.println("ho letto "+line);
	    client.add(st.nextToken().substring(1));
	    date.add(st.nextToken().substring(2));
	    request.add(st.nextToken().substring(2));
	    result.add(st.nextToken().substring(2));
	}
	nLines = client.size();
    }

    
    /**
     * Creates a HTML sorted log.
     * @param output  a name _without_ the .html extension. The generated file
     * will be named <output>X.html, where X = [1,4] depends of the sort mode
     * @param sortMode  the sort criteria chosen: an integer among
     * SORT_BY_CLIENT, SORT_BY_DATE, SORT_BY_REQUEST, SORT_BY_RESULT
     */    
    private void sort(String output, int sortMode) 
	                                  throws ParseException, IOException {
	int i, j, min;
	for (i = 0; i < nLines; i++) {
	    min = i;
	    for (j = i + 1; j < nLines; j++) {
		switch(sortMode) {
		case SORT_BY_CLIENT:
		    if (numValue((String) client.get(j)) <
			numValue((String) client.get(min))) min = j; 
		    break;
		case SORT_BY_DATE:
		    // deprecated as of JDK 1.1. It worked very well,
		    // but in some cases may cause problems and has been 
		    // eliminated.
		    // if (Date.parse((String) date.get(j)) < 
		    // Date.parse((String) date.get(min))) min = j;
		    break;
		case SORT_BY_REQUEST:
		    if (((String) request.get(j)).
			compareTo((String) request.get(min)) < 0) min = j;
		    break;
		case SORT_BY_RESULT:
		    if (((String) result.get(j)).
			compareTo((String) result.get(min)) < 0) min = j;
		    break;
		}
	    }
            swap(min,i);
	}
	makeHtml(output, sortMode);
    }


    /**
     * Swaps two elements in all the four ArrayLists.
     */ 
    private void swap(int index1, int index2) {
	Object dummy;
	dummy = client.set(index2, client.set(index1, client.get(index2)));
	dummy = date.set(index2, date.set(index1, date.get(index2)));
	dummy = request.set(index2, request.set(index1, request.get(index2)));
	dummy = result.set(index2, result.set(index1, result.get(index2)));
    }


    /**
     * Transforms an IP address in its numerical value, eg. giving
     * 193.55.61.191 as parameter will return 193055061191. Necessary
     * when sorting IP addresses.
     */
    public static long numValue(String IPAddress) {
	double IPValue = 0;
	StringTokenizer st = new StringTokenizer(IPAddress, ".");
	for (int l = 3; l >= 0; l--)
	    IPValue += Integer.parseInt(st.nextToken()) * 
		Math.pow((double) 1000, (double) l);
	return (long) IPValue;
    }


    /**
     * Generates an HTML document presenting the sorted log.
     * @param output  a name _without_ the .html extension. The generated file
     * will be named <output>X.html, where X = [1,4] depends of the sort mode
     * @param sortMode  the sort criteria chosen: an integer among
     * SORT_BY_CLIENT, SORT_BY_DATE, SORT_BY_REQUEST, SORT_BY_RESULT
     */
    private void makeHtml(String output, int sortMode) throws IOException {
	String sortModeText = "";
	String sortMode0Tag = "";
	String sortMode1Tag = "";
	String sortMode2Tag = "";
	String sortMode3Tag = "";
	String sortModeTag = " BGCOLOR=#EEEEEE";
	int n;
	
	switch(sortMode) {
	case SORT_BY_CLIENT: 
	    sortModeText = "sorted by IP address of client";
	    sortMode0Tag = new String(sortModeTag);
	    break;
	case SORT_BY_DATE: 
	    sortModeText = "sorted by date and time of the request";
	    sortMode1Tag = new String(sortModeTag);
	    break;
	case SORT_BY_REQUEST: 
	    sortModeText = "sorted by the request's start line";
	    sortMode2Tag = new String(sortModeTag);
	    break;
	case SORT_BY_RESULT: 
	    sortModeText = "sorted by the result of the request";
	    sortMode3Tag = new String(sortModeTag);
	    break;
	}

	FileWriter fw = new FileWriter(output + (sortMode+1) + ".html");
	fw.write("<HTML>\n<HEAD>\n<TITLE>" + filename + "</TITLE>\n</HEAD>\n");
	fw.write("<BODY BGCOLOR=\"#FFFF99\">\n");
	fw.write("<CENTER><H2>Log (" + filename + ")</H2>\n");
	fw.write("at " + filedate.toString() + "<BR>" + 
		 sortModeText + "<BR>" + nLines + " entries read" + 
		 "</CENTER><BR>\n");
	fw.write("<DIV ALIGN=RIGHT><H4><A HREF=\"control-panel.html\">" + 
		 "Control Panel</A></H4></DIV><BR><BR><BR>\n");
	fw.write("<TABLE ROWS=2 COLS=4 CELLPADDING=15><TR>\n");
	fw.write("<TD WIDTH=1" + sortMode0Tag + "><CENTER><I><A HREF=\"" + 
		 output + "1.html\">Client</A></I></CENTER></TD>\n");
	fw.write("<TD WIDTH=1" + sortMode1Tag + "><CENTER><I><A HREF=\"" + 
	     output + "2.html\">Date</A></I></CENTER></TD>\n");
	fw.write("<TD WIDTH=1" + sortMode2Tag + "><CENTER><I><A HREF=\"" + 
		 output + "3.html\">Request</A></I></CENTER></TD>\n");
	fw.write("<TD WIDTH=1" + sortMode3Tag + "><CENTER><I><A HREF=\"" + 
		 output + "4.html\">Result</A></I></CENTER></TD>\n");
	fw.write("</TR><TR>\n");
	fw.write("<TD NOWRAP>\n");
	for (n = 0; n < nLines; n++) {
	    fw.write((String) client.get(n) + "<BR>\n");
	}
	fw.write("</TD>\n");
	fw.write("<TD NOWRAP>\n");
	for (n = 0; n < nLines; n++) {
	    fw.write((String) date.get(n) + "<BR>\n");
	}
	fw.write("</TD>\n");
	fw.write("<TD NOWRAP>\n");
	for (n = 0; n < nLines; n++) {
	    fw.write((String) request.get(n) + "<BR>\n");
	}
	fw.write("</TD>\n");
	fw.write("<TD NOWRAP>\n");
	String resultElement;
	for (n = 0; n < nLines; n++) {
	    resultElement = (String) result.get(n);
	    if (resultElement.equals("ACCEPTED")) 
		fw.write("<FONT COLOR=green>");
	    else
		fw.write("<FONT COLOR=red>");
	    fw.write(resultElement + "</FONT><BR>\n");
	}
	fw.write("</TD>\n");
	fw.write("</TR></TABLE>\n</BODY>\n</HTML>\n");
	
	fw.close();
}
    
    
    /** 
     * Generates the sets of 4 sorted logs, one for every sort criteria.
     */ 
    public void generateSortedLogs(String output) 
	                                  throws ParseException, IOException {
	filedate = new Date();
	for (int g = 0; g < 4; g++) {
	    if (ProxyMain.TRACEON_LOG) 
		System.out.println("Generating sorted log " + output +
				   ", part " + (g+1) + " of 4...");
	    sort(output, g);
	}
    }


}



