import java.util.*;


/**
 * Main class.
 */
public class ProxyMain {

    // constant values
    public static final String CFGFILE = "proxy.cfg";
    public static final int CFG_CHECK_INTERVAL = 5;
    public static final int TIMEOUT = 30;

    // other runtime values
    public static String PROXY_HOST;
    public static Date PROXY_STARTTIME = new Date();

    // values loaded from configuration file
    public static int MIN_THREADS;
    public static int MIN_SPARE_THREADS;
    public static int MAX_THREADS;
    public static int MAX_SPARE_THREADS;
    public static int PROXY_PORT;
    public static String LOGFILE;
    public static boolean ENABLE_LOGGING;
    public static int CONTROLPANEL_UPDATE_DELAY;

    // flags for tracing events
    public static boolean TRACEON_LOG = false;  
    public static boolean TRACEON_THREADS = false;
    public static boolean TRACEON_FILTERS = false;
    public static boolean TRACEON_CONNECTIONS = false;
    public static boolean TRACEON_CONFIGURATION = false;
    public static boolean TRACEON_MSGS = false;
    public static boolean TRACEON_FULLMSGS = false;


    static AccessVerifier accessClient; 
    static AccessVerifier accessServer;
    static AuthorizationVerifier auth;
    static LogfileManager log;




    /**
     * Main application.
     * @param args  the options passed in command line
     */
    public static void main(String[] args) throws Exception {

	System.out.println(
    "RAMM Proxy v1.0   1 MAY 2001\n" +
    "by Daniele Raffo, Matteo Magnani and Ccile Michelou   UMLV\n" +
    "OSI Certified Open Source Software\n\n" +			   
    "This program is free software; you can redistribute it and/or modify\n" +
    "it under the terms of the GNU General Public License, version 2,\n" + 
    "as published by the Free Software Foundation.\n\n" +
    "This program is distributed in the hope that it will be useful,\n" +
    "but WITHOUT ANY WARRANTY; without even the implied warranty of\n" +
    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the\n" +
    "GNU General Public License for more details.\n\n"
    );

	if (args.length != 0) {
	    if (args[0].equals("--help")) {
		showHelp();
	    }
	    else { 
		if (args[0].startsWith("-")) {
		    StringBuffer unrecon = new StringBuffer("");
		    for (int i = 1; i < args[0].length(); i++) {
			switch (args[0].charAt(i)) {
			case 'l': TRACEON_LOG = true; break;
			case 't': TRACEON_THREADS = true; break;
			case 'f': TRACEON_FILTERS = true; break;
			case 'c': TRACEON_CONNECTIONS = true; break;
			case 'g': TRACEON_CONFIGURATION = true; break;
			case 'm': TRACEON_MSGS = true; break;
			case 'M': TRACEON_MSGS = true;
			          TRACEON_FULLMSGS = true; break;
			default: unrecon.append(args[0].charAt(i));  
			}
		    }
		    if (unrecon.length() != 0)
			System.out.println("Warning: unrecognized option(s) -"
					   + unrecon.toString());
		}
		else showHelp(); 
	    }
	}
	

	Configuration configuration = new Configuration(CFGFILE);

	accessClient = new AccessVerifier("in.cfg"); 
	accessServer = new AccessVerifier("out.cfg"); 
	auth         = new AuthorizationVerifier("aut.cfg");
	log          = new LogfileManager(LOGFILE);
      
	Proxy proxy = new Proxy();


    } // end of main()


    /**
     * Shows a screen of help about the program.
     */
    public static void showHelp() {
	System.out.println(
		  "Usage: java ProxyMain [TRACING OPTIONS]\n\n" +
		  "Options for tracing:\n" +
		  "-l     writings on logfile\n" +
		  "-t     thread operations\n" +
		  "-f     filtering\n" +
		  "-c     network connections\n" +
		  "-g     configuration changes\n" +
		  "-m     HTTP message headers\n" +
		  "-M     HTTP message headers and bodies (flood!)\n"
		  );
	System.exit(1);
    }


}
